#!/usr/local/bin/perl -w
use blib;
use strict;
use Geo::StormTracker::Parser;

my @lines=<DATA>;
my $advisory=join('',@lines);

my $parser=Geo::StormTracker::Parser->new();

my $adv_obj=$parser->read_data($advisory);

#my $result=join('--',@{$adv_obj->movement_toward()});
my $result=$adv_obj->stringify();

if (defined($result)){
	print "result is: \n",$result;
}
else {
	print "result is: undefined\n";
}

exit;

__DATA__
 
  
 
061 
WTNT31 KNHC 062041
TCPAT1
BULLETIN
TROPICAL DEPRESSION ELEVEN ADVISORY NUMBER   9
NATIONAL WEATHER SERVICE MIAMI FL
4 PM CDT WED OCT 06 1999
 
...TROPICAL DEPRESSION ELEVEN WEAKENS TO A BROAD TROUGH OF LOW
PRESSURE...
 
AT 4 PM CDT...2100Z...TROPICAL DEPRESSION ELEVEN HAS WEAKENED TO A
TROUGH WITH THE REMAINS OF THE CENTER LOCATED NEAR LATITUDE 21.0
NORTH...LONGITUDE  95.0 WEST OR ABOUT 150 MILES...240 KM...NORTH-
NORTHEAST OF VERACRUZ MEXICO.
 
ALTHOUGH THE CENTER OF THE DEPRESSION HAS DISSIPATED...REPORTS FROM
AN AIR FORCE RESERVE HURRICANE HUNTER AND SHIPS INDICATE THAT WINDS
HAVE INCREASED OVER PARTS OF THE SOUTHWESTERN GULF OF MEXICO. SEE
OFFSHORE FORECASTS FROM THE NATIONAL WEATHER SERVICE FORECAST OFFICE
NEW ORLEANS AND HIGH SEAS FORECASTS FROM THE TROPICAL PREDICTION
CENTER FOR DETAILS ON GALE WARNINGS.
 
HEAVY RAINS ARE LIKELY TO CONTINUE OVER PARTS OF EASTERN MEXICO.
THESE ARE LIKELY TO PRODUCE MORE LIFE-THREATENING FLASH FLOODS AND
MUDSLIDES.
 
THIS IS THE LAST PUBLIC ADVISORY ISSUED BY THE NATIONAL HURRICANE
CENTER ON THIS SYSTEM. THIS SYSTEM WILL BE CLOSELY MONITORED FOR
SIGNS OF REGENERATION.
 
BEVEN
 
NO STRIKE PROBABILITIES ARE ASSOCIATED WITH THIS ADVISORY.
 
 
 

